"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const element_1 = require("./element");
/**
 *
 * @param {*} node
 * @param {*} options
 */
function render(node, options) {
    const nodes = 'length' in node ? node : [node];
    let output = '';
    for (let i = 0; i < nodes.length; i++) {
        output += renderNode(nodes[i], options);
    }
    return output;
}
function renderNode(node, options) {
    switch (node.type) {
        case element_1.ElementType.Program:
            return render(node.body, options);
        case element_1.ElementType.WXComment:
            return renderComment(node);
        case element_1.ElementType.WXScript:
        case element_1.ElementType.WXElement:
            return renderElement(node, options);
        case element_1.ElementType.WXText:
            return renderText(node, options);
        case element_1.ElementType.WXInterpolation:
            return renderInterpolation(node);
        case element_1.ElementType.WXAttributeInterpolation:
            return renderWXAttributeInterpolation(node, options);
    }
}
function renderComment(node) {
    return `<!--${node.value}-->`;
}
function renderElement(node, options) {
    let tag = `<${node.name}`;
    const attribs = formatAttributes(node.startTag.attributes, options);
    if (attribs) {
        tag += ` ${attribs}`;
    }
    if (!node.endTag) {
        tag += ' />';
    }
    else {
        tag += '>';
        // WXScript
        if (node.value) {
            tag += node.value;
            // WXElement
        }
        else if (node.children) {
            tag += render(node.children);
        }
        tag += `</${node.name}>`;
    }
    return tag;
}
// 单双引号
function formatAttributes(attribs, options) {
    let printed = '';
    attribs.forEach((attrib, idx) => {
        printed += renderAttribute(attrib, options);
        if (idx !== attribs.length - 1)
            printed += ' ';
    });
    return printed;
}
function renderAttribute(attrib, options) {
    if (!attrib.quote)
        return attrib.key;
    let attribVal = '';
    if (Array.isArray(attrib.children) && attrib.children.length) {
        attrib.children.forEach(child => {
            attribVal += render(child, options);
        });
    }
    else {
        attribVal += attrib.value;
    }
    return attrib.key + '=' + attrib.quote + attribVal + attrib.quote;
}
function renderWXAttributeInterpolation(node, options) {
    return `{{${node.value}}}`;
}
// TODO: 测试决定是否需要补充，关于encodeEntities和xmlMode的逻辑
function renderText(node, options) {
    let value = node.value;
    // TODO: 这里关于encodeEntities和xmlMode的逻辑
    // if (options.encodeEntities) {
    //     value =
    // }
    return value;
}
function renderInterpolation(node) {
    return `{{${node.value}}}`;
}
exports.default = render;
